/* $Id: dumbfs.C,v 1.2 2001/04/26 03:24:59 dm Exp $ */

/*
 *
 * Copyright (C) 2001 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "mount_prot.h"
#include "dumbfs.h"
#include "rxx.h"
#include "nfsserv.h"
#include "nfsmounter.h"
#include "nfs3_prot.h"

str srvname;
in_addr srvaddr;
dirpath srvpath;
str mntpath;

ptr<axprt_unix> nmx;
ptr<aclnt> nmc;
ptr<nfsserv_udp> nfss = New refcounted<nfsserv_udp>;
ptr<aclnt> nfsc;
AUTH *rootauth = authunixint_create (myname (), 0, 0, 0, NULL);

static void
reply (nfscall *nc, clnt_stat stat)
{
  if (stat) {
    warn << "NFS server: " << stat << "\n";
    nc->reject (SYSTEM_ERR);
  }
  else
    nc->reply (nc->getvoidres ());
}

void
dispatch (nfscall *nc)
{
  static AUTH *ao = authopaque_create ();
  authopaque_set (ao, nc->getaup ());
  nfsc->call (nc->proc (), nc->getvoidarg (), nc->getvoidres (),
	      wrap (reply, nc), ao);
}

static void
mounted (ref<mountres> resp, clnt_stat err)
{
  if (err)
    fatal << "nfsmounter: " << err << "\n";
  if (resp->status)
    fatal << mntpath << ": " << strerror (resp->status) << "\n";
}

static void
gotfh (ref<mountres3> resp, clnt_stat err)
{
  if (err)
    fatal << srvname  << ": " << err << "\n";
  if (resp->fhs_status)
    fatal << srvpath << ": " << resp->fhs_status << "\n";

  mountarg arg;
  arg.hostname = srvname << "*";
  arg.path = mntpath;
  arg.flags = NMOPT_NFS3;
  arg.handle.set (resp->mountinfo->fhandle.base (),
		  resp->mountinfo->fhandle.size ());
  ref<mountres> mrp (New refcounted<mountres>);
  nmx->sendfd (nfss->getfd (), false);
  nmc->call (NFSMOUNTER_MOUNT, &arg, mrp, wrap (mounted, mrp));
}

static void
gotnfsc (str host, ptr<aclnt> c, clnt_stat err)
{
  if (err || !c)
    fatal << host << ": no NFS3 server running\n";
  nfsc = c;
  nfss->setcb (wrap (dispatch));
  ref<mountres3> resp (New refcounted<mountres3>);
  acallrpc (srvaddr, mount_program_3, MOUNTPROC3_MNT,
	    &srvpath, resp, wrap (gotfh, resp), 0, rootauth);
}

static void
usage ()
{
  warnx << "usage: " << progname << " host:/dir /mountpoint\n";
  exit (1);
}

int
main (int argc, char **argv)
{
  setprogname (argv[0]);
  if (argc != 3)
    usage ();

  rxx hostrx ("([\\w-]+):(/.*)");
  if (!hostrx.match (argv[1]) || argv[2][0] != '/')
    usage ();
  srvname = hostrx[1];
  srvpath = hostrx[2];
  mntpath = argv[2];

  str nmpath = fix_exec_path ("nfsmounter");
  if (!nmpath)
    fatal ("cannot find nfsmounter daemon\n");
  nmx = axprt_unix_spawn (nmpath, 0, "nfsmounter", mntpath.cstr (), NULL);
  if (!nmx)
    fatal ("%s: %m\n", nmpath.cstr ());
  nmc = aclnt::alloc (nmx, nfsmounter_prog_1);

  if (hostent *h = gethostbyname (srvname))
    srvaddr = *(in_addr *) h->h_addr;
  else
    fatal << srvname << ": hostname lookup failed\n";

  aclntudp_create (srvaddr, 0, nfs_program_3, wrap (gotnfsc, hostrx[1]));

  amain ();
}
